# Generated by roxygen2: do not edit by hand

export("getSampleID<-")
export("getTreatment<-")
export(PCASamples)
export(adjust.methylC)
export(adjustMethylC)
export(annotate.WithFeature)
export(annotate.WithFeature.Flank)
export(annotate.WithGenicParts)
export(assocComp)
export(bedgraph)
export(calculateDiffMeth)
export(calculateDiffMethDSS)
export(clusterSamples)
export(dataSim)
export(diffMethPerChr)
export(filterByCoverage)
export(get.methylDiff)
export(getAssembly)
export(getContext)
export(getCorrelation)
export(getCoverageStats)
export(getDBPath)
export(getData)
export(getFeatsWithTargetsStats)
export(getFlanks)
export(getMembers)
export(getMethylDiff)
export(getMethylationStats)
export(getSampleID)
export(getTargetAnnotationStats)
export(getTreatment)
export(joinSegmentNeighbours)
export(makeMethylDB)
export(methRead)
export(methSeg)
export(methSeg2bed)
export(methylRawList)
export(methylRawListDB)
export(normalizeCoverage)
export(percMethylation)
export(plotTargetAnnotation)
export(pool)
export(processBismarkAln)
export(read)
export(read.bed)
export(read.bismark)
export(read.feature.flank)
export(read.transcript.features)
export(readMethylDB)
export(reconstruct)
export(regionCounts)
export(removeComp)
export(reorganize)
export(select)
export(selectByOverlap)
export(tileMethylCounts)
export(unite)
exportClasses(methylBase)
exportClasses(methylBaseDB)
exportClasses(methylDiff)
exportClasses(methylDiffDB)
exportClasses(methylRaw)
exportClasses(methylRawDB)
exportClasses(methylRawList)
exportClasses(methylRawListDB)
import(GenomicRanges)
import(emdbook)
import(methods)
import(qvalue)
importClassesFrom(IRanges,RangedSelection)
importClassesFrom(S4Vectors,Annotated)
importClassesFrom(S4Vectors,DataFrame)
importClassesFrom(S4Vectors,List)
importClassesFrom(S4Vectors,SimpleList)
importClassesFrom(S4Vectors,Vector)
importClassesFrom(S4Vectors,character_OR_NULL)
importFrom(Seqinfo,'seqlevels<-')
importFrom(IRanges,IRanges)
importFrom(IRanges,RangedSelection)
importFrom(IRanges,countOverlaps)
importFrom(IRanges,findOverlaps)
importFrom(IRanges,ranges)
importFrom(R.utils,isGzipped)
importFrom(Rcpp,sourceCpp)
importFrom(Rsamtools,"yieldSize<-")
importFrom(Rsamtools,TabixFile)
importFrom(Rsamtools,scanTabix)
importFrom(Rsamtools,yieldSize)
importFrom(S4Vectors,"elementMetadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,Rle)
importFrom(S4Vectors,elementMetadata)
importFrom(S4Vectors,levels)
importFrom(S4Vectors,subjectHits)
importFrom(data.table,":=")
importFrom(data.table,CJ)
importFrom(data.table,SJ)
importFrom(data.table,as.data.table)
importFrom(data.table,copy)
importFrom(data.table,data.table)
importFrom(data.table,fread)
importFrom(data.table,haskey)
importFrom(data.table,key)
importFrom(data.table,rbindlist)
importFrom(data.table,setcolorder)
importFrom(data.table,setkey)
importFrom(data.table,setkeyv)
importFrom(data.table,setnames)
importFrom(data.table,setorder)
importFrom(data.table,tables)
importFrom(fastseg,fastseg)
importFrom(grDevices,colorRamp)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,densCols)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(grDevices,topo.colors)
importFrom(graphics,abline)
importFrom(graphics,barplot)
importFrom(graphics,boxplot)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,pairs)
importFrom(graphics,par)
importFrom(graphics,pie)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(graphics,smoothScatter)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(gtools,chr)
importFrom(gtools,mixedsort)
importFrom(limma,squeezeVar)
importFrom(mclust,Mclust)
importFrom(mclust,densityMclust)
importFrom(parallel,mclapply)
importFrom(rtracklayer,export.bed)
importFrom(stats,IQR)
importFrom(stats,as.dendrogram)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coefficients)
importFrom(stats,cor)
importFrom(stats,cor.test)
importFrom(stats,dendrapply)
importFrom(stats,dhyper)
importFrom(stats,dist)
importFrom(stats,dnorm)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,glm.fit)
importFrom(stats,hclust)
importFrom(stats,is.leaf)
importFrom(stats,kruskal.test)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,optimize)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,phyper)
importFrom(stats,pnorm)
importFrom(stats,prcomp)
importFrom(stats,quantile)
importFrom(stats,rbeta)
importFrom(stats,rnbinom)
importFrom(stats,uniroot)
importFrom(stats,wilcox.test)
importFrom(utils,head)
importFrom(utils,read.table)
importFrom(utils,write.table)
importMethodsFrom(IRanges,as.data.frame)
importMethodsFrom(IRanges,nearest)
importMethodsFrom(Seqinfo,seqlengths)
importMethodsFrom(Seqinfo,seqlevels)
importMethodsFrom(Seqinfo,seqnames)
importMethodsFrom(Seqinfo,sortSeqlevels)
useDynLib(methylKit)
