#' Example methylRawList object.
#' 
#'
#'  \code{methylBase}, \code{methylDiff} and \code{methylRawList}.
#'   You can load the data using \code{data(methylKit)}
#'
#' @name methylRawList.obj
#' @docType data
#' @format  Methylation data from multiple the samples regardless 
#'   of common coverage are stored in  methylRawList.obj object. 
#'   \code{methylRawList} extends \code{list} S3 class
#'
NULL
#' Example methylBase object.
#' 
#'  \code{methylBase}, \code{methylDiff} and \code{methylRawList}. 
#'  You can load the data using \code{data(methylKit)}
#'
#' @name methylBase.obj
#' @docType data
#' @format methylBase.obj object stores the location and methylation 
#' information for bases that
#'   are covered in all samples. \code{methylBase} partially extends 
#'   \code{data.frame} S3 class.
#'
NULL
#' Example methylKit objects.
#' 
#'  \code{methylBase}, \code{methylDiff} and \code{methylRawList}. 
#'  You can load the data using \code{data(methylKit)}
#'
#' @name methylDiff.obj
#' @docType data
#' @format The Differential methylation information is stored in 
#' methylDiff.obj object.
#'   \code{methylBase} partially extends \code{data.frame} S3 class.
#'
NULL
