# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

methCall <- function(read1, type = "bam", nolap = FALSE, minqual = 20L, mincov = 10L, phred64 = FALSE, CpGfile = "", CHHfile = "", CHGfile = "", verbosity = 1L) {
    invisible(.Call('_methylKit_methCall', PACKAGE = 'methylKit', read1, type, nolap, minqual, mincov, phred64, CpGfile, CHHfile, CHGfile, verbosity))
}

